#version 150

// vertexPosition was bound to attribute index 0 and vertexColor was bound to attribute index 1
in vec3 vertexPosition;
in vec3 vertexNormal;
in vec4 vertexColor;
in vec2 vertexTextureUV;

// Our outputs the next shader (fragment) in the chain
out vec4 fragmentColor;
out vec2 fragmentTextureUV;

out vec3 fsDirectionVS;
out vec3 fsPositionEye;
out vec3 fsNormalVS;
out vec3 fsNormalWS;
out vec3 fsSunDirectionVS;
out mat4 fsInvView;

uniform mat4 mvpMatrix;
uniform mat4 modelViewMatrix;
uniform mat4 modelMatrix;
uniform mat4 viewMatrix;
uniform mat4 sunCameraView;
uniform mat4 sunCameraProjection[4];

//Expected to be a unit vector and in world space already. Pointing FROM origin TO the sun.
uniform vec4 sunLightDirection = vec4(-0.447766751, 0.373415232, 0.812444329, 0.422158539);

void main(void)
{
	gl_Position = vec4(vertexPosition, 1.0) * mvpMatrix;

	fsNormalWS = vec3(normalize(vec4(vertexNormal, 0.0) * modelMatrix));
	fragmentColor = vertexColor;
	fragmentTextureUV = vertexTextureUV;

	mat4 modelSunView = modelMatrix * sunCameraView;

	fsPositionEye = (vec4(vertexPosition, 1.0) * modelViewMatrix).xyz;

	fsNormalVS = fsNormalWS * mat3(viewMatrix);
	fsSunDirectionVS = (vec4(vec3(sunLightDirection), 0.0) * viewMatrix).xyz;

	fsInvView = inverse(viewMatrix);

	// vec3 mirror_dir = reflect(-view_dir, normal);
	// mirror_dir = vec3(inverse(viewMatrix) * vec4(mirror_dir, 0.0));

	// mat3 normalMatrix = modelMatrix; /* this will break for non-uniform scaling */
	// vec3 normalW = normalize(vertexNormal * normalMatrix);
	// vec3 tangentW = normalize(vec3(modelMatrix * vec4(a_tangent.xyz, 0.0)));
	// vec3 bitangentW = cross(normalW, tangentW) * a_tangent.w;
	// fsTbnMatrix = mat3(tangentW, bitangentW, normalW);

	vec4 wpos = vec4(vertexPosition, 1.0) * modelMatrix;
	vec4 vpos = wpos * viewMatrix;
	fsDirectionVS = -vpos.xyz;
}
