#version 150

// This came from some tutorial, mentioning some drivers required this to function
//   properly but did not delve into why. 
precision highp float;

in  vec3 fragmentNormal;
in  vec4 fragmentColor;
in  vec2 fragmentTextureUV;
///For shadowed particles...
in vec4 shadowCoordinates[4];

out vec4 finalFragColor;

// diffuseTexture is a 2D texture sampler for diffuse texture.
uniform sampler2D diffuseTexture;
uniform vec4 ambientLightColor = vec4(1.0, 1.0, 1.0, 1.0);
uniform vec4 sunLightColor = vec4(1.0, 1.0, 1.0, 1.0);
uniform vec4 sunLightDirection = vec4(-0.447766751, 0.373415232, 0.812444329, 0.422158539);  //Expected to be a unit vector and in world space already.
uniform vec4 materialColor = vec4(1.0, 1.0, 1.0, 1.0);
//These uniforms control the bias of each cascaded map.
uniform int poissonSamples = 8;
uniform sampler2DShadow shadowTexture[4];
uniform vec4 poissonScales = vec4(1500.0, 2000.0, 2250.0, 2500.0);
uniform vec4 shadowBias = vec4(0.0, 0.0, 0.0001, 0.0002);


void main(void)
{
	float shadowFactor = 1.0;
	for (int i = 0; i < 4; ++i)
	{
		vec4 biasedShadowCoordinate = shadowCoordinates[i];
		biasedShadowCoordinate.z -= shadowBias[i];

		vec2 texCoord = biasedShadowCoordinate.xy / biasedShadowCoordinate.w;
		bool outside = any(greaterThan(texCoord.xy, vec2(0.995, 0.995))) || any(lessThan(texCoord.xy, vec2(0.005, 0.005)));
		if (false == outside)
		{
			shadowFactor = textureProj(shadowTexture[i], biasedShadowCoordinate);
			break;
		}
	}

	float nDotL = abs(dot(fragmentNormal, vec3(sunLightDirection)));
	//float nDotL = abs(dot(fragmentNormal, vec3(sunLightDirection)));
	//fragmentColor = vec4(nDotL * vec3(vertexColor), 1.0);

	vec4 textureColor = texture(diffuseTexture, fragmentTextureUV);
	vec4 premulMaterialColor = vec4(materialColor.rgb, 1.0) * (textureColor.a * fragmentColor.a);
	vec3 finalColor = vec3(fragmentColor * premulMaterialColor * textureColor) * max(0.225, shadowFactor);

	float gamma = 2.2;
	finalFragColor = vec4(pow(finalColor, vec3(1.0 / gamma, 1.0 / gamma, 1.0 / gamma)), textureColor.a * fragmentColor.a);

	//finalFragColor = vec4(finalFragColor.rgb, 1.0) * (texturedFragmentColor.a);

	if (finalFragColor.a < 0.05)
	{
		discard;
	}
}
