#version 150

// vertexPosition was bound to attribute index 0 and vertexColor was bound to attribute index 1
in vec3 vertexPosition;
in vec3 vertexNormal;
in vec4 vertexColor;
in vec2 vertexTextureUV;
 
// Our outputs the next shader (fragment) in the chain
out vec3 fragmentNormal;
out vec4 fragmentColor;
out vec2 fragmentTextureUV;

uniform mat4 projectionMatrix;
uniform mat4 viewMatrix;

//For shadowed particles
out vec4 shadowCoordinates[4];

//Expected to be a unit vector and in world space already. Pointing FROM origin TO the sun.
uniform vec4 sunLightDirection = vec4(-0.447766751, 0.373415232, 0.812444329, 0.422158539);
uniform vec4 shadowNormalOffset = vec4(0.05);
uniform mat4 sunCameraProjection[4];
uniform mat4 sunCameraView;
uniform mat4 modelMatrix;

const mat4 bias = mat4(
	vec4(0.5, 0.0, 0.0, 0.5),
	vec4(0.0, 0.5, 0.0, 0.5),
	vec4(0.0, 0.0, 0.5, 0.5),
	vec4(0.0, 0.0, 0.0, 1.0)
);
/////////////////////////


struct InstanceData
{
	vec4 position; //w = scale
	vec4 rotation; //w = unknown
	vec4 color;    //rgba
};

layout (std140) uniform Particle //NOT A STRUCT
{
	InstanceData instances[128];
};

void main(void)
{
	InstanceData particle = instances[gl_InstanceID];

	mat4 scale = mat4(
		particle.position.w, 0.0, 0.0, 0.0,
		0.0, particle.position.w, 0.0, 0.0,
		0.0, 0.0, particle.position.w, 0.0,
		0.0, 0.0, 0.0, 1.0);

	mat4 translate = mat4(
		1.0, 0.0, 0.0, particle.position.x,
		0.0, 1.0, 0.0, particle.position.y,
		0.0, 0.0, 1.0, particle.position.z,
		0.0, 0.0, 0.0, 1.0);

	mat4 rotx = mat4(
		1.0, 0.0, 0.0, 0.0, 
		0.0, sin(particle.rotation.x), cos(particle.rotation.x), 0.0,
		0.0, cos(particle.rotation.x), -sin(particle.rotation.x), 0.0,
		0.0, 0.0, 0.0, 1.0);

	mat4 roty = mat4(
		sin(particle.rotation.y), 0.0, cos(particle.rotation.y), 0.0,
		0.0, 1.0, 0.0, 0.0,
		cos(particle.rotation.y), 0.0, -sin(particle.rotation.y), 0.0,
		0.0, 0.0, 0.0, 1.0);

	mat4 rotz = mat4(
		sin(particle.rotation.z), cos(particle.rotation.z), 0.0, 0.0,
		cos(particle.rotation.z), -sin(particle.rotation.z), 0.0, 0.0,
		0.0, 0.0, 1.0, 0.0,
		0.0, 0.0, 0.0, 1.0);

	mat4 newModelMatrix = rotx * roty * rotz * scale * translate;


	gl_Position = vec4(vertexPosition, 1.0) * newModelMatrix * viewMatrix * projectionMatrix;

	fragmentNormal = vec3(normalize(vec4(vertexNormal, 0.0) * newModelMatrix));
	fragmentColor = vec4(particle.color.rgb, 1.0) * (particle.color.a);
	fragmentTextureUV = vertexTextureUV;

	///// For particles with shadows
	mat4 modelSunView = newModelMatrix * sunCameraView;
	for (int i = 0; i < 4; i++)
	{
		float shadowOffset = max(0.0, dot(fragmentNormal, sunLightDirection.xyz)) * shadowNormalOffset[i];
		vec4 positionOffset = vec4(vertexPosition + fragmentNormal * shadowOffset, 1.0f) * modelSunView;
		shadowCoordinates[i] = positionOffset * sunCameraProjection[i] * bias;
	}
}
